/*************************************************************************
 *  Project Name:    FFT
 *  FileName:        demo_fft_main.c
 *  Dependencies:    xc.h, stdint.h, math.h, fft.h
 *  Processor:       PIC24, dsPIC
 *  Compiler:        XC16
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description: Demo code for the FFT library functions.
 *
 * written by Anton Alkhimenok 04/02/2015
 *************************************************************************/

#include <xc.h>
#include <stdint.h>
#include <math.h>
#include "fft.h"

// PIC24FJ128GA010 CONFIGURATION SETTINGS
_CONFIG2(POSCMOD_HS & FNOSC_PRIPLL)
_CONFIG1(WDTPS_PS128 & FWPSA_PR128 & WINDIS_ON & FWDTEN_OFF & ICS_PGx2 & JTAGEN_OFF)

#define PIx2  ((double)2*3.14159265)

// arrays for input/output FFT
int re[FFT_POINTS_NUMBER];
int im[FFT_POINTS_NUMBER];	// imaginary part must be zero for the real signal

// arrays for harmonics' amplitudes and phases
// only harmonics from 0(DC) to FFT_POINTS_NUMBER/2 are valid
unsigned amplitude[FFT_POINTS_NUMBER/2];
int phase[FFT_POINTS_NUMBER/2];

int main(void) {
 unsigned i;
    // load the arrays with a sum of sine waves
 for (i=0; i<FFT_POINTS_NUMBER; i++) {
  re[i] = (int)(
   2048+ // DC amplitude 2048
   8192*cos(PIx2*i/(FFT_POINTS_NUMBER/2)+(PIx2*10/360))+ // 2nd harmonic amplitude 8192, phase 10 deg
   8192*sin(PIx2*i/(FFT_POINTS_NUMBER/4))+ // 4th harmonic amplitude 8192, phase -90 deg (sin(x) = cos(x-90deg))
   8000*cos(PIx2*i/(FFT_POINTS_NUMBER/7))  // 7th harmonic amplitude 8000, phase 0 deg
  );
 }
 FFT(re,im);
    // calculate harmonics amplitudes and phases
    // only harmonics from 0(DC) to FFT_POINTS_NUMBER/2 are valid
 for (i=0; i<FFT_POINTS_NUMBER/2; i++) {
        // DC amplitude in the frequency dimension must match the amplitude
        // in time dimension (2048/1 = 2048)
        // Harmonics amplitudes in the frequency dimension must be half of
        // amplitudes in time dimension (8192/2 = 4096)
  amplitude[i] = FFTAmplitude(re[i],im[i]);
  phase[i] = FFTPhase(re[i],im[i]);
 }
 return i;
}
